"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyViewModelToElement = exports.extendViewModel = exports.ViewModel = exports.isNullOrUndefined = exports.extend = void 0;
const knockout_1 = require("knockout");
const userAgent_1 = require("./userAgent");
const event_1 = require("./event");
const style_1 = require("./style");
function extend(deep, ...objects) {
    var extended = {};
    var withChildren = false;
    let i = 0;
    const length = arguments.length;
    if (typeof (arguments[0]) === "boolean") {
        withChildren = arguments[0];
        i++;
    }
    const merge = obj => {
        for (let prop in obj) {
            if (Object.prototype.hasOwnProperty.call(obj, prop)) {
                if (withChildren && Object.prototype.toString.call(obj[prop]) === "[object Object]") {
                    extended[prop] = extend(true, extended[prop], obj[prop]);
                }
                else {
                    extended[prop] = obj[prop];
                }
            }
        }
    };
    for (; i < length; i++) {
        const obj = arguments[i];
        merge(obj);
    }
    return extended;
}
exports.extend = extend;
function isNullOrUndefined(value) {
    return value === null || typeof value === "undefined";
}
exports.isNullOrUndefined = isNullOrUndefined;
class ViewModel {
    constructor(options) {
        this.visible = true;
        this.disabled = false;
        this.addViewModelToDataObject = false;
        options = options || {};
        this.classNames = options.classNames;
        this.attributes = options.attributes || {};
        this.style = options.style || {};
        if (options.hasOwnProperty("visible")) {
            this.visible = options.visible;
        }
        if (options.hasOwnProperty("disabled")) {
            this.disabled = options.disabled;
        }
        this.innerHtml = options.innerHtml;
        this.innerText = options.innerText;
        this.addViewModelToDataObject = options.addViewModelToDataObject;
        this.userSelectable = options.userSelectable;
        this.dispose = options.dispose;
        this.refresh = options.refresh;
        this.eventListeners = options.eventListeners || {};
        this.viewType = options.viewType;
    }
    getStyle(...cssProperties) {
        if (cssProperties && this.style) {
            const style = {};
            cssProperties.forEach((cssProperty) => {
                style[cssProperty] = this.style[cssProperty];
            });
            return style;
        }
        return null;
    }
    getStyleValue(cssProperty) {
        if (this.style) {
            return (0, knockout_1.unwrap)(this.style[cssProperty]);
        }
        return null;
    }
    addMouseEventListener(event, mouseEventListener) {
        if (!this.eventListeners[event]) {
            this.eventListeners[event] = mouseEventListener;
        }
    }
    addEventListener(event, genericEventListener) {
        if (!this.eventListeners[event]) {
            this.eventListeners[event] = genericEventListener;
        }
    }
    setClasses(classNames, replace) {
        if ((0, knockout_1.isSubscribable)(this.classNames)) {
            var currentClassNames = (0, knockout_1.unwrap)(this.classNames);
            if (replace) {
                currentClassNames = new Array();
            }
            if (classNames) {
                classNames.forEach((className) => {
                    currentClassNames.push(className);
                });
            }
            this.classNames(currentClassNames);
        }
    }
    setAttributes(attributes) {
        if (attributes) {
            Object.keys(attributes).forEach((key) => {
                let newValue = attributes[key];
                if (newValue === null || newValue === undefined) {
                    newValue = null;
                }
                if ((0, knockout_1.isSubscribable)(this.attributes[key])) {
                    this.attributes[key]((0, knockout_1.unwrap)(newValue));
                }
                else {
                    this.attributes[key] = newValue;
                }
            });
        }
    }
    setStyle(style) {
        if (style) {
            Object.keys(style).forEach((key) => {
                let newValue = style[key];
                if (newValue === null || newValue === undefined) {
                    if (userAgent_1.userAgent.browser.isInternetExplorer) {
                        newValue = style_1.defaultStyle[key];
                    }
                    else {
                        newValue = null;
                    }
                }
                if ((0, knockout_1.isSubscribable)(this.style[key])) {
                    this.style[key]((0, knockout_1.unwrap)(newValue));
                }
                else {
                    this.style[key] = newValue;
                }
            });
        }
    }
    setVisible(visible) {
        if ((0, knockout_1.isSubscribable)(this.visible)) {
            this.visible(visible);
        }
    }
    setDisabled(disabled) {
        if ((0, knockout_1.isSubscribable)(this.disabled)) {
            this.disabled(disabled);
        }
    }
    setInnerHtml(innerHtml) {
        if ((0, knockout_1.isSubscribable)(this.innerHtml)) {
            this.innerHtml(innerHtml);
        }
    }
    setInnerText(innerText) {
        if ((0, knockout_1.isSubscribable)(this.innerText)) {
            this.innerText(innerText);
        }
    }
    removeClasses(classNames) {
        if ((0, knockout_1.isSubscribable)(this.classNames)) {
            const currentClassNames = (0, knockout_1.unwrap)(this.classNames);
            for (let i = 0; i < currentClassNames.length; i++) {
                if (classNames.includes(currentClassNames[i])) {
                    currentClassNames.splice(i, 1);
                }
            }
            this.classNames(currentClassNames);
        }
    }
    focus() {
    }
}
exports.ViewModel = ViewModel;
function extendViewModel(target, source) {
    return extend(true, target, source);
}
exports.extendViewModel = extendViewModel;
function applyViewModelToElement(viewModel, element) {
    element.setAttribute("viewtype", viewModel.viewType || "View");
    if (viewModel.classNames) {
        applyClassesToElement((0, knockout_1.unwrap)(viewModel.classNames), true, element);
        if ((0, knockout_1.isSubscribable)(viewModel.classNames)) {
            viewModel.classNames.subscribe((classNames) => {
                applyClassesToElement(classNames, true, element);
            });
        }
    }
    if (viewModel.attributes) {
        Object.keys(viewModel.attributes).forEach((key) => {
            var value = viewModel.attributes[key];
            applyAttributeToElement(key, (0, knockout_1.unwrap)(value), element);
            if ((0, knockout_1.isSubscribable)(value)) {
                value.subscribe((newValue) => {
                    applyAttributeToElement(key, newValue, element);
                });
            }
        });
    }
    if (viewModel.style) {
        Object.keys(viewModel.style).forEach((key) => {
            const style = viewModel.style[key];
            element.style[key] = (0, knockout_1.unwrap)(style);
            if ((0, knockout_1.isSubscribable)(style)) {
                style.subscribe((newStyle) => {
                    element.style[key] = newStyle;
                });
            }
        });
    }
    if (!isNullOrUndefined(viewModel.innerHtml)) {
        element.innerHTML = (0, knockout_1.unwrap)(viewModel.innerHtml);
        if ((0, knockout_1.isSubscribable)(viewModel.innerHtml)) {
            viewModel.innerHtml.subscribe((innerHtml) => {
                element.innerHTML = innerHtml;
            });
        }
    }
    if (!isNullOrUndefined(viewModel.innerText)) {
        element.innerText = (0, knockout_1.unwrap)(viewModel.innerText);
        if ((0, knockout_1.isSubscribable)(viewModel.innerText)) {
            viewModel.innerText.subscribe((innerText) => {
                element.innerText = innerText;
            });
        }
    }
    if (viewModel.userSelectable === false) {
        applyEventListenerToElement("selectstart", () => { return false; }, { passive: true }, element);
        element.style.userSelect = "none";
    }
    if (viewModel.addViewModelToDataObject) {
        if (!element["data"]) {
            element["data"] = {};
        }
        element["data"].viewModel = viewModel;
    }
    if (viewModel.eventListeners) {
        applyEventListenersToElement(viewModel, element);
    }
    applyVisibilityToElement((0, knockout_1.unwrap)(viewModel.visible), element);
    if ((0, knockout_1.isSubscribable)(viewModel.visible)) {
        viewModel.visible.subscribe((visible) => {
            applyVisibilityToElement(visible, element);
        });
    }
    applyDisabledToElement((0, knockout_1.unwrap)(viewModel.disabled), element);
    if ((0, knockout_1.isSubscribable)(viewModel.disabled)) {
        viewModel.disabled.subscribe((disabled) => {
            applyDisabledToElement(disabled, element);
        });
    }
    viewModel.focus = () => {
        element.focus();
    };
}
exports.applyViewModelToElement = applyViewModelToElement;
function applyVisibilityToElement(visible, element) {
    if (visible === false) {
        element.classList.add("hidden");
    }
    else {
        element.classList.remove("hidden");
    }
}
function applyDisabledToElement(disabled, element) {
    if (disabled) {
        element.setAttribute("disabled", "disabled");
    }
    else {
        element.removeAttribute("disabled");
    }
}
function applyEventListenersToElement(viewModel, element) {
    const clickEventListeners = new Array();
    const doubleClickEventListeners = new Array();
    Object.entries(viewModel.eventListeners).forEach((entry) => {
        var eventName = entry[0];
        var eventListener = entry[1];
        if (eventName === "click") {
            clickEventListeners.push(eventListener);
            return;
        }
        if (eventName === "dblclick") {
            doubleClickEventListeners.push(eventListener);
            return;
        }
        if (eventName === "dragstart") {
            element.setAttribute("draggable", "true");
        }
        if ((0, event_1.isGlobalEvent)(eventName)) {
            applyEventListenerToElement(eventName, (event) => {
                if (eventListener.options) {
                    const options = eventListener.options;
                    const keyEvent = event;
                    if (options.altKey && !keyEvent.altKey) {
                        return;
                    }
                    if (options.shiftKey && !keyEvent.shiftKey) {
                        return;
                    }
                    if (options.ctrlKey && !keyEvent.ctrlKey) {
                        return;
                    }
                    if (options.eventKey && options.eventKey !== keyEvent.key) {
                        return;
                    }
                }
                eventListener.listener.call(viewModel, event);
            }, eventListener.options, element);
        }
        else if (event_1.mouseEvents[eventName] && userAgent_1.userAgent.device.supportsMouseEvents) {
            applyEventListenerToElement(eventName, eventListener.listener, eventListener.options, element);
        }
        else if (event_1.touchEvents[eventName] && userAgent_1.userAgent.device.supportsTouchEvents) {
            applyEventListenerToElement(eventName, eventListener.listener, eventListener.options, element);
        }
    });
    clickEventListeners.forEach((clickEventListener) => {
        if (userAgent_1.userAgent.device.supportsTouchEvents) {
            applyEventListenerToElement("tap", clickEventListener.listener, clickEventListener.options, element);
        }
        if (userAgent_1.userAgent.device.supportsMouseEvents) {
            applyEventListenerToElement("click", clickEventListener.listener, clickEventListener.options, element);
        }
    });
    doubleClickEventListeners.forEach((doubleClickEventListener) => {
        if (userAgent_1.userAgent.device.supportsTouchEvents) {
            applyEventListenerToElement("dbltap", doubleClickEventListener.listener, doubleClickEventListener.options, element);
        }
        if (userAgent_1.userAgent.device.supportsMouseEvents) {
            applyEventListenerToElement("dblclick", doubleClickEventListener.listener, doubleClickEventListener.options, element);
        }
    });
}
function applyEventListenerToElement(type, listener, options, element) {
    if (!type) {
        throw new Error("type is mandatory");
    }
    if (!listener) {
        throw new Error("listener is mandatory");
    }
    if (!options) {
        options = {};
    }
    if (!options.passive) {
        options.passive = false;
    }
    if (!options.once) {
        options.once = false;
    }
    if (element.addEventListener) {
        element.addEventListener(type, (event) => {
            listener(event);
        }, {
            passive: options.passive,
            once: options.once
        });
    }
    else if (element["attachEvent"]) {
        element["attachEvent"](type, (event) => {
            listener(event);
        });
    }
    else {
        throw "Your browser does not support 'addEventListener'";
    }
}
function applyClassesToElement(classNames, replace, element) {
    if (replace) {
        element.className = "";
    }
    if (classNames) {
        classNames.forEach((className) => {
            element.classList.add(className);
        });
    }
}
function applyAttributeToElement(name, value, element) {
    if (isNullOrUndefined(value)) {
        element.removeAttribute(name);
    }
    else {
        element.setAttribute(name, value);
    }
}
//# sourceMappingURL=ViewModel.js.map